/*
    Copyright (C) 2005 Andreas Sandberg <andreas@sandberg.pp.se>
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "oss2jack.h"

/* This method both creates and, if memory allocations succeed, 
   adds an oss_instance to the list of availible instances.
*/
struct oss_instance* oss_create(struct device_info* info, pid_t pid,
				struct dsp_instance* dsp,
				struct mixer_instance* mixer)
{
	struct oss_instance* instance = malloc(sizeof(struct oss_instance));
	if(instance == NULL)
		return NULL;
	
	instance->pid = pid;
	instance->dsp = dsp;
	instance->mixer = mixer;
	
	instance->next = info->instances;
	info->instances = instance;
	
	return instance;

}

/* Tries to lookup an instance based on a PID, returns NULL on failure */
struct oss_instance* oss_find_pid(struct device_info* info, pid_t pid)
{
	struct oss_instance* instance = info->instances;
	
	while(instance != NULL) {
		if(instance->pid == pid)
			return instance;
		
		instance = instance->next;
	}
	
	return NULL;
}

void oss_delete(struct device_info* info, struct oss_instance* instance)
{
	struct oss_instance* i = info->instances;
	
	while(i != NULL)
	{
		if(i->next == instance) {
			i->next = instance->next;
			free(instance);
			return;
		}
		
		i = i->next;
	}
	
	if(info->instances == instance) {
		info->instances = instance->next;
		free(instance);
	}
}
