/*
    Copyright (C) 2004 Kor Nielsen
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include FUSD_H_PATH
#include <sys/soundcard.h>
#include <samplerate.h>
#include <jack/jack.h>
#include <sys/types.h>


#define SUPPORTED_FORMATS AFMT_S16_LE | AFMT_S16_BE | AFMT_S16_NE | AFMT_S8 | AFMT_U8;

struct circular_buffer
{
	int buffer_size;
	int alloc_size;
	int dma;
	short* buffer;
	short* wptr;
	short* rptr;
};

struct mixer_instance
{
	int volume[2];
};

struct dsp_instance
{
	int sample_format;
	int num_channels;
	int sample_rate;
	
	int out_total_samples_mixed;
	
	int out_enabled;
	
	//current write info
	short* out_samples;
	short* out_sample_ptr;
	int out_samples_left_to_mix;
	int out_write_size;
	
	int frag_size;
	int frag_count;
	
	struct circular_buffer out;
	struct circular_buffer in;
	
	int in_total_samples_read;
	
	short* in_samples;
	short* in_sample_ptr;
	int in_samples_left_to_read;
	int in_read_size;
	int in_enabled;
	
	int can_block;
	
	double out_rate_ratio;
	double in_rate_ratio;
	
	SRC_STATE* out_rate_converter;
	SRC_STATE* in_rate_converter;
	
	struct fusd_file_info* in_file_info;
	struct fusd_file_info* out_file_info;
	
	struct fusd_file_info* polldiff;
};

struct oss_instance
{
	pid_t pid;
	
	struct mixer_instance* mixer;
	struct dsp_instance* dsp;

	struct oss_instance* next;
};

struct device_info
{
	jack_client_t* client;
	jack_port_t* output_ports[2];
	jack_port_t* input_ports[2];
	int sample_rate;
	
	int volume[2];
	int rec_volume[2];
	int mix_modcount;

	pthread_mutex_t mutex;
	int samples_sent;
	int debuglevel;
	int last_frame_size;
	volatile int restart_interval;
	
	struct oss_instance* instances;
};

// Define the DEBUG_LEVEL values used for printfs
#define DEBUG_LEVEL_CRITICAL 0
#define DEBUG_LEVEL_ERROR 1
#define DEBUG_LEVEL_WARNING 2
#define DEBUG_LEVEL_NOTICE 3
#define DEBUG_LEVEL_INFO 4
#define DEBUG_LEVEL_DEBUG1 5
#define DEBUG_LEVEL_DEBUG2 6
#define DEBUG_LEVEL_DEBUG3 7
#define DEBUG_LEVEL_DEBUG4 8

// Define the returnvalues mentionned above in help message:
#define ERR_SUCCESS 0
#define ERR_BADARG 1
#define ERR_JACKD_FAIL 2
#define ERR_FUSD_FAIL 3
#define ERR_DETACH_FAIL 4
#define ERR_JACKD_EXIT 5

// Define the maximum value for device number
#define MAX_DEVICE_NUMBER 99

inline void pprintf(const int level, const char *format, ...);
inline void pperror(const int level, const char *format, ...);

int cb_get_count(struct circular_buffer* cbuf);
int cb_get_available(struct circular_buffer* cbuf);
int cb_set_volume(struct circular_buffer* cbuf,
		  int vol_left, int vol_right,
		  float* left, float* right,
		  int num_samples);
int cb_read(struct circular_buffer* cbuf, float* samples, int num_samples);
int cb_read_short(struct circular_buffer* cbuf, short* samples, int num_samples);
int cb_read_stereo_mix(struct circular_buffer* cbuf, float* left, float* right, float vol_l, float vol_r, int num_samples);
int cb_read_mono_mix(struct circular_buffer* cbuf, float* left, float* right, float vol_l, float vol_r, int num_samples);
int cb_skip(struct circular_buffer* cbuf, int num_samples);
int cb_write(struct circular_buffer* cbuf, float* samples, int count);
int cb_write_short(struct circular_buffer* cbuf, short* s, int count);
int cb_write_stereo(struct circular_buffer* cbuf, float* left, float* right, float vol_l, float vol_r, int count);
int cb_write_mono(struct circular_buffer* cbuf, float* left, float* right, float vol_l, float vol_r, int count);


int do_dsp_open(struct fusd_file_info* file);
int do_dsp_close(struct fusd_file_info* file);
int do_dsp_read(struct fusd_file_info* file, char* buffer, size_t size, loff_t* offset);
int do_dsp_write(struct fusd_file_info *file, const char *buffer, size_t size, loff_t *offset);
void do_dsp_notify_complete(struct oss_instance* instance, struct device_info* dinfo);
ssize_t do_dsp_notify_polldiff(struct fusd_file_info* file, unsigned int cached_state);
int do_dsp_mmap(struct fusd_file_info *file, int offset, size_t length, int flags, void** addr, size_t* out_length);
int do_dsp_ioctl(struct fusd_file_info *file, int request, void *data);

int do_mixer_open(struct fusd_file_info* file);
int do_mixer_close(struct fusd_file_info* file);
int do_mixer_ioctl(struct fusd_file_info *file, int request, void *data);

struct oss_instance* oss_create(struct device_info* info, pid_t pid,
				struct dsp_instance* dsp,
				struct mixer_instance* mixer);
struct oss_instance* oss_find_pid(struct device_info* info, pid_t pid);
struct oss_instance* oss_find_pid(struct device_info* info, pid_t pid);


struct fusd_file_operations dsp_file_ops;
struct fusd_file_operations mixer_file_ops;
